axios = require('axios')

class Powerwall {
  constructor (host, password) {
    this.host = host
    this.password = password

    this.tokenPromise = null
  }

  getToken () {
    if (!this.tokenPromise) {
      this.tokenPromise = axios.post(`http://${this.host}/api/login/Basic`,
        {
          'username': '',
          'password': this.password,
          'force_sm_off': false,
        }
      ).then((response) => {return response.data['token']})
    }
    return this.tokenPromise
  }

  getReq () {
    return this.getToken().then((token) => {
      return axios.create({
        baseURL: `http://${this.host}`,
        headers: {
          'Authorization': `Bearer ${token}`
        }
      })
    })
  }

  _getOperation () {
    return this.getReq().then((req) => {
      return req.get(`/api/operation`).then((response) => {return response.data})
    })
  }

  getOperation () {
    return this._getOperation()
      .catch((error) => {
        if (error.response.status === 401) {
          this.tokenPromise = null
          return this._getOperation()
        }
      })
  }

  _setOperation (reserve, mode = 'self_consumption') {
    return this.getReq().then((req) => {
      return req.post(`/api/operation`,
        {
          'mode': mode,
          'backup_reserve_percent': reserve
        }).then((response) => {return response.data})
    })
  }

  setOperation (reserve, mode = 'self_consumption') {
    return this._setOperation(reserve, mode)
      .catch((error) => {
        if (error.response.status === 401) {
          this.tokenPromise = null
          return this._setOperation(reserve, mode)
        }
      })
  }

  _commitOperation () {
    return this.getReq().then((req) => {
      return req.get('/api/config/completed').then((response) => response.data)
    })
  }

  commitOperation () {
    return this._commitOperation()
      .catch((error) => {
        if (error.response.status === 401) {
          this.tokenPromise = null
          this._commitOperation()
        }
      })
  }
}

module.exports = Powerwall